import { Row, RowIdentififying } from './util';
import { Table, DataTable, ContextChanges } from './table';
import { SettingsUpdateAction, TransactionAction, CommitContextAction, DBDispatch, RevertContextAction } from './actions';
import { Query } from './query';
export interface Data {
    [key: string]: DataTable<Row>;
}
export interface StateDefining {
    settings: {
        [key: string]: unknown;
    };
    data: Data;
}
export interface ContextState {
    _context?: {
        [context: string]: {
            [table: string]: ContextChanges<Row>;
        };
    };
}
export declare type RowKeyOf<T extends StateDefining> = Extract<keyof T['data'], string>;
export declare type RowType<State extends StateDefining, Key extends RowKeyOf<State>> = State['data'][Key]['byId']['someKey'];
export declare type SettingsKey<T extends StateDefining> = Extract<keyof T['settings'], string>;
export declare type SettingsType<State extends StateDefining, Key extends SettingsKey<State>> = State['settings'][Key];
export declare class DB<State extends StateDefining> {
    private state;
    private currentContext?;
    constructor(state: State, options?: {
        context?: string;
    });
    get<K extends SettingsKey<State>>(name: K): SettingsType<State, K>;
    set<K extends SettingsKey<State>, U extends SettingsType<State, K>>(name: K, value: U): SettingsUpdateAction;
    table<K extends RowKeyOf<State>>(type: K): Table<RowType<State, K>>;
    query<Key extends RowKeyOf<State>>(type: Key): Query<State, Key, RowType<State, Key>>;
    context(context: string): DB<State>;
    transaction(execute: (dispatch: DBDispatch) => void): TransactionAction;
    commit<K extends RowKeyOf<State>>(table?: K, ids?: RowIdentififying): CommitContextAction;
    revert<K extends RowKeyOf<State>>(table?: K, ids?: RowIdentififying): RevertContextAction;
    private changeSetsOfContext;
}
